/************************************************************************
 Author: Jingcao Hu  (jingcao@ece.cmu.edu)
 File name: worm_sim.cpp
 Date Created:  <Mon Oct 13 11:44:30 2003>
 Last Modified: <Mon Feb 14 15:27:01 2005>
 Description: 
************************************************************************/

#include <iostream>
#include <sys/types.h>
#include <cstdlib>
#include <cstdio>
#include <sys/times.h>

#include "version.hpp"
#include "misc.hpp"
#include "channel.hpp"
#include "pkt.hpp"
#include "routing_engine.hpp"
#include "version.hpp"
#include "msg.hpp"
#include "util.hpp"

using namespace std;

Net_clock net_clock;
Param param;

unsigned int Traffic_source::id_base = 0;
unsigned int Traffic_sink::id_base = 0;

int main(int argn, char ** argv) {

    parse_options(argn, argv);

    if (param.parse_error) {
        cerr << ERRO_PARSING_OPTIONS;
        return -1;
    }

    if (param.quit_flag)
        return 0;

    srand48(param.rand_seed);

    Network network;
    if (!network.get_success_flag()) {
        cerr << ERRO_NETWORK_BUILD_ERROR;
        return -1;
    }

    for (vector<char*>::iterator f = param.config_files.begin();
         f < param.config_files.end(); f++) {
        if (apply_config_file(*f)) {
            cerr << ERRO_CONFIG_FILE;
            return -1;
        }
    }
        
    if (post_initialization_configure(network)) {
        cerr << ERRO_NETWORK_BUILD_ERROR;
        return -1;
    }

//     if (param.analyze_latency) {
//         cout << "Perform latency analysis ... " << endl;
//         double latency = network.analyze_latency(); 
//         if (latency > 0)
//             cout << "Average latency for a packet: " << latency << endl;
//         else 
//             cout << "Average latency for a packet: Inf" << endl;
//         return 1;
//     }

    if (param.equation_file) {
        network.dump_equation_file();
        return 1;
    }

//     if (param.perf_anal_lib_equation_file) {
//         network.dump_perf_anal_lib_equation_file();
//         return 1;
//     }

    if (param.config_file_to_dump) {
        network.dump_config_file();
        return 1;
    }

    clock_t start, end;
    struct tms tmsstart, tmsend;
    start = times(&tmsstart);

    net_clock.reset();
    while (net_clock.tick() < param.simulation_length) {
        network.tick();
    }

    end = times(&tmsend);

    cout << INFO_SIM_DONE;

    print_parameters();
    print_statistics(network);

    if (param.print_simulation_time)
        print_times(end-start, &tmsstart, &tmsend);

    return 1;
}
